@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2023
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -14,12 +15,11 @@
 --------------------------------------------------------------------------------
 */
 
+#pragma once
 /** @file EquivalenceRelation.h
     @brief Implementation of polymake::EquivalenceRelation class
  */
 
-#ifndef POLYMAKE_EQUIVALENCE_RELATION_H
-#define POLYMAKE_EQUIVALENCE_RELATION_H
 
 #include "polymake/Array.h"
 #include "polymake/PowerSet.h"
@@ -48,10 +48,10 @@ namespace pm {
 class EquivalenceRelation {
 
 protected:
-   Array<int> equiv_classes;
-   hash_set<int> set_rep; 
-   Set<int> the_representatives;
-   std::list<int> erased_rep;
+   mutable Array<Int> equiv_classes;
+   hash_set<Int> set_rep; 
+   Set<Int> the_representatives;
+   std::list<Int> erased_rep;
    mutable bool dirty;
 
 public:
@@ -60,35 +60,35 @@ public:
 
    /// Creates the equivalence relation on the integers 0,..,@a size-1 where
    /// each element is contained in an equivalence class by itself.
-   explicit EquivalenceRelation(const int size)
-      : equiv_classes(size,sequence(0,size).begin())
-      , the_representatives(sequence(0,size))
+   explicit EquivalenceRelation(Int size)
+      : equiv_classes(size, sequence(0, size).begin())
+      , the_representatives(sequence(0, size))
       , dirty(false) { }
 
    /// Creates the equivalence relation on the integers 0,..,@a size-1 where
    /// each element is contained in an equivalence class by itself.  The @a
    /// representatives will be used as user defined representatives.
    template <typename Container>
-   EquivalenceRelation(const int size, const Container& represent)
-      : equiv_classes(size,sequence(0,size).begin())
-      , the_representatives(sequence(0,size))
+   EquivalenceRelation(Int size, const Container& represent)
+      : equiv_classes(size, sequence(0, size).begin())
+      , the_representatives(sequence(0, size))
       , dirty(true)
    {
-      for (auto v=entire(represent); !v.at_end(); ++v)
+      for (auto v = entire(represent); !v.at_end(); ++v)
          set_rep.insert(*v);
    }
 
 protected:
    template <typename Set>
-   void add_class(const GenericSet<Set, int>& the_class)
+   void add_class(const GenericSet<Set, Int>& the_class)
    {
       if (the_class.top().empty()) return;
-      const int highest=the_class.top().back();
+      const Int highest = the_class.top().back();
       if (equiv_classes.size() <= highest)
          equiv_classes.resize(highest+1, -1);
-      const int the_rep=the_class.top().front();
+      const Int the_rep = the_class.top().front();
       the_representatives += the_rep;
-      for (auto el=entire(the_class); !el.at_end(); ++el)
+      for (auto el = entire(the_class); !el.at_end(); ++el)
          equiv_classes[*el]=the_rep;
    }
 
@@ -105,22 +105,22 @@ public:
    }
 
    /// Return the representative of the equivalence class containing @a e.
-   int representative(int e) const
+   Int representative(Int e) const
    {
       if (POLYMAKE_DEBUG) {
-         if (e<0 || e>=equiv_classes.size())
+         if (e < 0 || e >= equiv_classes.size())
             throw std::runtime_error("EquivalenceRelation::representative - element out of range");
       }
       if (e == equiv_classes[e])
          return e;
 
-      int root = e;
-      std::list<int> update_root;
-      for ( ; equiv_classes[root]!=root; root=equiv_classes[root])
+      Int root = e;
+      std::list<Int> update_root;
+      for ( ; equiv_classes[root] != root; root = equiv_classes[root])
          update_root.push_back(root);
 
       while (!update_root.empty()) {
-         const_cast<Array<int>&>(equiv_classes)[update_root.front()] = root;  // keeping the representation neat
+         equiv_classes[update_root.front()] = root;  // keeping the representation neat
          update_root.pop_front();
       }
 
@@ -130,23 +130,23 @@ public:
    // ensures, that the pointer of each element points directly to its representative  
    void squeeze() const
    {
-      for (int i=0; i<equiv_classes.size(); ++i)
+      for (Int i = 0; i < equiv_classes.size(); ++i)
          representative(i);
-      dirty=false;
+      dirty = false;
    }
    
    /// Return the representatives of the equivalence classes of all elements.
-   const Array<int>& representatives() const
+   const Array<Int>& representatives() const
    {
       if (dirty) squeeze();
       return equiv_classes;
    }
    
    /// Merges two equivalence classes each represented by one (arbitrary) of its elements.
-   void merge_classes(const int c1, const int c2)
+   void merge_classes(const Int c1, const Int c2)
    {
-      int root1 = representative(c1);
-      int root2 = representative(c2);
+      Int root1 = representative(c1);
+      Int root2 = representative(c2);
       if (root1 == root2) return;
       if (root1 > root2) std::swap(root1, root2);
 
@@ -157,18 +157,18 @@ public:
          equiv_classes[root2] = root1;
          erased_rep.push_back(root2);
       }
-      dirty=true;
+      dirty = true;
    }
    
    /// Merges the classes of all the elements contained in classes.
    template <typename Container>
    void merge_classes(const Container& classes)
    {
-      auto c=entire(classes);
+      auto c = entire(classes);
       if (!c.at_end()) {
-         const int c0=*c;
+         const Int c0 = *c;
          for (++c; !c.at_end(); ++c)
-            merge_classes(c0,*c);
+            merge_classes(c0, *c);
       }
    }
 
@@ -182,19 +182,19 @@ public:
    }
 
    /// Return the set of all representatives.
-   const Set<int>& set_of_rep() const
+   const Set<Int>& set_of_rep() const
    {
       const_cast<EquivalenceRelation*>(this)->update_rep();
       return the_representatives;
    }
 
    /// Return the equivalence class of the element @a e.
-   Set<int> equivalence_class(const int e) const
+   Set<Int> equivalence_class(const Int e) const
    {
-      Set<int> class_of_e;
-      const int rep = representative(e);
+      Set<Int> class_of_e;
+      const Int rep = representative(e);
 
-      for (int i=0; i<equiv_classes.size(); ++i)
+      for (Int i = 0; i < equiv_classes.size(); ++i)
          if (rep == representative(i))
             class_of_e += i;
 
@@ -202,50 +202,50 @@ public:
    }
 
    /// Return all equivalence classes.
-   PowerSet<int> equivalence_classes() const
+   PowerSet<Int> equivalence_classes() const
    {
-      PowerSet<int> classes;
-      hash_map< int,Set<int> > rep_map;
+      PowerSet<Int> classes;
+      hash_map<Int, Set<Int>> rep_map;
 
-      for (int i=0, n=equiv_classes.size(); i<n; ++i)
+      for (Int i = 0, n = equiv_classes.size(); i < n; ++i)
          rep_map[representative(i)] += i;
 
       for (const auto& cl : rep_map)
          classes += cl.second;
 
-      dirty=false;
+      dirty = false;
       return classes;
    }
 
    /// Return true iff @a e1 and @a e2 are in the same equivalence class.
-   bool related(const int e1, const int e2) const
+   bool related(const Int e1, const Int e2) const
    {
-      return representative(e1)==representative(e2);
+      return representative(e1) == representative(e2);
    }
    
    /// The element @a e will be used as user defined representative of it's equivalence class.
-   void set_representative(const int e)
+   void set_representative(const Int e)
    {
       set_rep.insert(e);
-      const int root = representative(e);
+      const Int root = representative(e);
 
-      if (root!=e) {  // adjust representatives
+      if (root != e) {  // adjust representatives
          equiv_classes[root] = e;
          equiv_classes[e] = e;
          update_rep();
          the_representatives -= root;
          the_representatives += e;
-         dirty=true;
+         dirty = true;
       }
    }
 
    template <typename Input>
    friend Input& operator>> (GenericInput<Input>& is, EquivalenceRelation& me)
    {
-      PowerSet<int> classes;
+      PowerSet<Int> classes;
       is.top() >> classes;
       me.clear();
-      for (auto cl=entire(classes); !cl.at_end(); ++cl)
+      for (auto cl = entire(classes); !cl.at_end(); ++cl)
          me.add_class(*cl);
       return is.top();
    }
@@ -254,7 +254,7 @@ public:
 template <>
 struct spec_object_traits< EquivalenceRelation >
    : spec_object_traits<is_container> {
-   typedef PowerSet<int> serialized;
+   typedef PowerSet<Int> serialized;
 
    static serialized serialize(const EquivalenceRelation& ER)
    {
@@ -268,7 +268,6 @@ namespace polymake {
    using pm::EquivalenceRelation;
 }
 
-#endif // POLYMAKE_EQUIVALENCE_RELATION_H
 
 
 // Local Variables:
