@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,12 +15,11 @@
 --------------------------------------------------------------------------------
 */
 
+#pragma once
 /** @file GenericGraph.h
     @brief Implementation of pm::GenericGraph class
 */
 
-#ifndef POLYMAKE_GENERIC_GRAPH_H
-#define POLYMAKE_GENERIC_GRAPH_H
 
 #include "polymake/GenericIncidenceMatrix.h"
 #include "polymake/GenericMatrix.h"
@@ -804,7 +803,6 @@ namespace polymake {
    using pm::AdjacencyMatrix;
 }
 
-#endif // POLYMAKE_GENERIC_GRAPH_H
 
 // Local Variables:
 // mode:C++
