@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2019
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,14 +15,15 @@
 --------------------------------------------------------------------------------
 */
 
+#pragma once
 #include "polymake/GenericStruct.h"
 
 namespace polymake { namespace topaz {
 
 struct IntersectionForm : GenericStruct<IntersectionForm> {
-   DeclSTRUCT( DeclFIELD(parity,int)
-               DeclFIELD(positive,int)
-               DeclFIELD(negative,int) );
+   DeclSTRUCT( DeclFIELD(parity, Int)
+               DeclFIELD(positive, Int)
+               DeclFIELD(negative, Int) );
 };
 
 } }
