@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2023
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -18,7 +19,6 @@ use namespaces;
 use warnings qw(FATAL void syntax misc);
 
 package Polymake::Core::Rule;
-use POSIX qw( :signal_h :sys_wait_h );
 
 use Polymake::Enum Flags => {
    is_function => 0x1,            #  does not return anything, can be called arbitrarily many times
@@ -50,8 +50,8 @@ use Polymake::Struct (
    '@output',                           # ( prop_ref )
    [ '$flags' => '0' ],                 # is_*
    [ '$code' => '#2' ],                 # sub { ... }  compiled perl code of the rule body
-   [ '$defined_for' => '#3' ],          # ObjectType
-   [ '$overridden_in' => 'undef' ],     # [ ObjectType ... ]
+   [ '$defined_for' => '#3' ],          # BigObjectType
+   [ '$overridden_in' => 'undef' ],     # [ BigObjectType ... ]
    '@labels',                           # rule labels
    '$weight',                           # [ weight_category, weight_value ]
    '@preconditions',                    # ( Rule )
@@ -114,7 +114,7 @@ sub parse_input {
    my ($self, $sources, $seen)=@_;
    @{$self->input}=map {
       my $input=$self->defined_for->encode_read_request($_);
-      if ($Application::plausibility_checks) {
+      if ($enable_plausibility_checks) {
          check_repeated_properties($_, $seen, 1) for @$input;
       }
       $input
@@ -135,13 +135,13 @@ sub parse_output {
       my $any_mult_depth=0;
 
       foreach (split /\./, $target) {
-         if (my ($prop_name, $attrib)= /^($id_re) (?: \(\s* ($id_re) \s*\) )? $/xo) {
+         if (my ($prop_name, $attrib)= /^($prop_name_re) (?: \(\s* ($prop_name_re) \s*\) )? $/xo) {
             $prop=$proto->property($prop_name);
             $proto=$prop->type;
             push @out, $prop;
             if ($prop->flags & Property::Flags::is_multiple) {
                if (defined $attrib) {
-                  if ($Application::plausibility_checks) {
+                  if ($enable_plausibility_checks) {
                      foreach my $output (@{$self->output}) {
                         my $l=Property::equal_path_prefixes(\@out, $output);
                         if ($l==@out) {
@@ -176,13 +176,13 @@ sub parse_output {
                   $array_flag |= Property::Flags::is_multiple;
                }
             } else {
-               if ($Application::plausibility_checks && defined($attrib)) {
+               if ($enable_plausibility_checks && defined($attrib)) {
                   $attrib eq "new" || $attrib eq "any"
                   ? croak( "Attribute '$attrib' is only applicable to multiple subobject properties" )
                   : croak( "Unknown property attribute '$attrib'" );
                }
                if ($prop->flags & Property::Flags::is_permutation) {
-                  if ($Application::plausibility_checks && $array_flag == Property::Flags::is_multiple_new) {
+                  if ($enable_plausibility_checks && $array_flag == Property::Flags::is_multiple_new) {
                      croak( "Rule dealing with a permutation may not create new instances of multiple subobjects" );
                   }
                   $array_flag |= Property::Flags::is_permutation;
@@ -193,7 +193,7 @@ sub parse_output {
          }
       }
 
-      if ($Application::plausibility_checks) {
+      if ($enable_plausibility_checks) {
          if ($prop->flags & Property::Flags::is_multiple) {
             croak( "A rule may not create a multiple subobject as a whole but only its single properties" );
          }
@@ -228,7 +228,7 @@ sub source_location {
 #
 #  Constructor - compile the rule
 #
-#  new Rule(header, \&body, ObjectType);
+#  new Rule(header, \&body, BigObjectType);
 #
 sub new {
    my $self=&_new;
@@ -236,7 +236,7 @@ sub new {
    my ($is_nonexistent);
    if (defined $self->code) {
       @parts=split /\s*:\s*/, $self->header, 3;
-      if ($Application::plausibility_checks and @parts < 2 || $parts[-1] =~ /:/) {
+      if ($enable_plausibility_checks and @parts < 2 || $parts[-1] =~ /:/) {
          croak( "ill-formed rule header" );
       }
 
@@ -282,7 +282,16 @@ sub new {
    parse_input($self, $parts[1], \%seen);
    parse_output($self, $parts[0], \%seen);
 
-   if ($Application::plausibility_checks && !defined($self->code)) {
+   # to print warnings for plain subobjects as rule-targets
+   #   if ($enable_plausibility_checks) {
+   #      foreach my $output (@{$self->output}) {
+   #         if (($output->[-1]->flags & (Property::Flags::is_subobject | Property::Flags::is_subobject_array)) == Property::Flags::is_subobject) {
+   #            warn_print( "rule target ", Property::print_path($output), " is a subobject which is deprecated; at ", $self->source_location );
+   #         }
+   #      }
+   #   }
+
+   if ($enable_plausibility_checks && !defined($self->code)) {
       if (@{$self->input}>1 || @{$self->output}>1) {
          croak( "A shortcut rule must have exactly one source and one target" );
       }
@@ -303,7 +312,7 @@ sub analyze_spez_preconditions {
 #
 #  Constructor for special rules
 #
-#  special Rule 'Header', \&body, ObjectType
+#  special Rule 'Header', \&body, BigObjectType
 #
 sub special {
    my $self=&_new;
@@ -324,7 +333,7 @@ sub append_precondition {
    $precond->header="precondition " . $precond->header . " ( " . $self->header . " )";
    push @{$self->preconditions}, $precond;
    if ($override) {
-      if ($Application::plausibility_checks) {
+      if ($enable_plausibility_checks) {
          $self->flags & Flags::in_restricted_spez
            or croak( "useless or repeated `override precondition'" );
       }
@@ -350,7 +359,7 @@ sub append_weight {
    my ($self, $major, $minor, $spec) = @_;
    if (defined $major) {
       $self->weight = [ $major, $minor ];
-      assign_max($max_major, $major + 1); # reserve the highest category for preference violation penalties
+      assign_max($max_major, $major+1); # reserve the highest category for preference violation penalties
    } else {
       $self->weight = $zero_weight;
    }
@@ -363,9 +372,9 @@ sub append_weight {
 }
 
 sub append_permutation {
-   my ($self, $perm_name)=@_;
-   my @perm_path=$self->defined_for->encode_descending_path($perm_name);
-   if ($Application::plausibility_checks) {
+   my ($self, $perm_name) = @_;
+   my @perm_path = $self->defined_for->encode_descending_path($perm_name);
+   if ($enable_plausibility_checks) {
       if (Property::find_first_in_path(\@perm_path, Property::Flags::is_permutation) != $#perm_path) {
          croak( "$perm_name is not a permutation" );
       }
@@ -435,18 +444,18 @@ sub flip_rule_for_twin {
 }
 ####################################################################################
 sub needs_finalization {
-   my ($self)=@_;
+   my ($self) = @_;
 
    my @path_to_perm;
    foreach my $input (@{$self->input}) {
       if (get_array_flags($input) & Property::Flags::is_permutation) {
-         if ($Application::plausibility_checks) {
+         if ($enable_plausibility_checks) {
             if ($self->flags & Flags::is_initial) {
                croak( "initial rules may not trigger permutations" );
             }
             foreach my $path (@$input) {
-               my $perm_index=Property::find_first_in_path($path, Property::Flags::is_permutation);
-               if ($perm_index<0) {
+               my $perm_index = Property::find_first_in_path($path, Property::Flags::is_permutation);
+               if ($perm_index < 0) {
                   croak( "A permuted subobject mixed with non-permuted properties in one source group" );
                }
                if (@path_to_perm) {
@@ -454,7 +463,7 @@ sub needs_finalization {
                      croak( "Different permuted subobjects occur as sources in the same rule" );
                   }
                }
-               @path_to_perm=@$path[0..$perm_index];
+               @path_to_perm = @$path[0..$perm_index];
                if (grep { $_->flags & Property::Flags::is_permutation } @$path[$perm_index+1..$#$path]) {
                   croak( "Nested permuted subobjects are not supported" );
                }
@@ -472,28 +481,28 @@ sub needs_finalization {
          return;
       }
    } else {
-      croak( "Only permutation rules may be labeled as nonexistent") if $self->code==\&nonexistent;
+      croak( "Only permutation rules may be labeled as nonexistent") if $self->code == \&nonexistent;
       push @{$self->defined_for->production_rules}, $self;
    }
    $self
 }
 ####################################################################################
 sub finalize {
-   my ($self, $for_twin_rule)=@_;
-   my $proto=$self->defined_for;
-   my $perm_deputy=$self->with_permutation;
-   my $perm_path=$perm_deputy && $perm_deputy->perm_path;
+   my ($self, $for_twin_rule) = @_;
+   my $proto = $self->defined_for;
+   my $perm_deputy = $self->with_permutation;
+   my $perm_path = $perm_deputy && $perm_deputy->perm_path;
 
    if ($self->flags & Flags::is_initial) {
-      if (defined $perm_deputy) {
+      if (defined($perm_deputy)) {
          die "rule ", $self->header, " can't trigger permutations at ", $self->source_location, "\n";
       }
-      $proto->add_producers_of(ObjectType::init_pseudo_prop()->key, $self);
+      $proto->add_producers_of(BigObjectType::init_pseudo_prop()->key, $self);
 
       foreach my $input (@{$self->input}) {
          foreach my $input_path (grep { @$_ > 1 } @$input) {
-            (undef, my @ancestors)=reverse(@$input_path);
-            my $prod_key=ObjectType::init_pseudo_prop()->get_prod_key(@ancestors);
+            (undef, my @ancestors) = reverse(@$input_path);
+            my $prod_key = BigObjectType::init_pseudo_prop()->get_prod_key(@ancestors);
             $proto->add_producers_of($prod_key, $self);
          }
       }
@@ -516,13 +525,13 @@ sub finalize {
    my $i=0;
    foreach my $output (@{$self->output}) {
       my ($created_prop, @prod);
-      if (@$output==1) {
-         $created_prop=$output->[0];
-         if ($perm_path && @$perm_path==1) {
-            if (defined (my $prod_list=$perm_path->[0]->sensitive_props->{$created_prop->key})) {
-               my $action=new PermAction($self, $i, $prod_list, 0);
+      if (@$output == 1) {
+         $created_prop = $output->[0];
+         if ($perm_path && @$perm_path == 1) {
+            if (defined(my $recovery = $perm_path->[0]->sensitive_props->{$created_prop->key})) {
+               my $action = new PermAction($self, $i, $recovery, 0);
                push @{$perm_deputy->actions}, $action;
-               push @prod, $action if @$prod_list;
+               push @prod, $action if $recovery != $Permutation::is_non_recoverable;
             } else {
                push @prod, $perm_deputy;
             }
@@ -530,64 +539,64 @@ sub finalize {
          $proto->add_producers_of($created_prop->key, $self, @prod);
 
       } else {
-         my ($prop, @ancestors)=reverse(@$output);
-         $created_prop=$prop;
+         my ($prop, @ancestors) = reverse(@$output);
+         $created_prop = $prop;
 
-         if ($perm_path && Property::equal_path_prefixes($perm_path, $output)==(my $perm_depth=$#$perm_path)) {
-            my $permutation=$perm_path->[-1];
-            my $sub_perm_hash=$permutation->sub_permutations;
+         if ($perm_path && Property::equal_path_prefixes($perm_path, $output) == (my $perm_depth = $#$perm_path)) {
+            my $permutation = $perm_path->[-1];
+            my $sub_perm_hash = $permutation->sub_permutations;
             local splice @$output, 0, $perm_depth;
-            my $last=@$output-1;
+            my $last = @$output-1;
             my $sub_perm_exists;
-            for (my $depth=1; $depth<=$last; ++$depth) {
-               if (defined (my $sub_permutation=$sub_perm_hash->{$output->[$depth-1]->key})) {
-                  if (defined (my $prod_list=
-                               $depth==$last ? $sub_permutation->sensitive_props->{$output->[$depth]->key}
-                                             : $sub_permutation->find_sensitive_sub_property(@$output[$depth..$last]))) {
-                     my $action=new PermAction($self, $i, $prod_list, $depth+$perm_depth, $sub_permutation);
+            for (my $depth=1; $depth <= $last; ++$depth) {
+               if (defined(my $sub_permutation = $sub_perm_hash->{$output->[$depth-1]->key})) {
+                  if (defined(my $recovery =
+                              $depth == $last ? $sub_permutation->sensitive_props->{$output->[$depth]->key}
+                                              : $sub_permutation->find_sensitive_sub_property(@$output[$depth..$last]))) {
+                     my $action = new PermAction($self, $i, $recovery, $depth+$perm_depth, $sub_permutation);
                      push @{$perm_deputy->actions}, $action;
-                     push @prod, $action;
-                     $sub_perm_exists=1;
+                     push @prod, $action if $recovery != $Permutation::is_non_recoverable;
+                     $sub_perm_exists = true;
                      last;
                   }
                }
-               defined( $sub_perm_hash=$sub_perm_hash->{$Permutation::sub_key} ) &&
-               defined( $sub_perm_hash=$sub_perm_hash->{$output->[$depth-1]->key} )
+               defined($sub_perm_hash = $sub_perm_hash->{$Permutation::sub_key}) &&
+               defined($sub_perm_hash = $sub_perm_hash->{$output->[$depth-1]->key})
                or last;
             }
             unless ($sub_perm_exists) {
-               if (defined (my $prod_list=$permutation->find_sensitive_sub_property(@$output))) {
-                  my $action=new PermAction($self, $i, $prod_list, $perm_depth, $perm_depth && $permutation);
+               if (defined(my $recovery = $permutation->find_sensitive_sub_property(@$output))) {
+                  my $action = new PermAction($self, $i, $recovery, $perm_depth, $perm_depth && $permutation);
                   push @{$perm_deputy->actions}, $action;
-                  push @prod, $action;
+                  push @prod, $action if $recovery != $Permutation::is_non_recoverable;
                } else {
                   push @prod, $perm_deputy;
                }
             }
          }
 
-         my $flags=get_array_flags($output);
-         my $any_mult_depth=$flags >> $any_mult_depth_shift;
+         my $flags = get_array_flags($output);
+         my $any_mult_depth = $flags >> $any_mult_depth_shift;
          my $common_prefix;  # the longest initial subpath of output occurring among inputs
          my $bad_twin;
 
          if ($flags & Property::Flags::is_permutation) {
             # rules dealing with permutations may not create new subobjects: pretend a full match
-            $common_prefix=$#$output;
+            $common_prefix = $#$output;
          } else {
             if (!$for_twin_rule && $ancestors[-1]->flags & Property::Flags::is_twin) {
-               $twin_targets{$ancestors[-1]}=1;
-               $bad_twin=1;
+               $twin_targets{$ancestors[-1]} = true;
+               $bad_twin = true;
             }
-            if ($any_mult_depth==0 || $Application::plausibility_checks) {
-               $common_prefix=0;
+            if ($any_mult_depth==0 || $enable_plausibility_checks) {
+               $common_prefix = 0;
                foreach my $rule ($self, @{$self->preconditions}) {
                   foreach my $input (@{$rule->input}) {
                      foreach my $input_path (@$input) {
                         my $l=Property::equal_path_prefixes($output, $input_path);
                         assign_max($common_prefix, $l);
-                        if ($Application::plausibility_checks) {
-                           $bad_twin &&= $l>0 if $rule==$self;
+                        if ($enable_plausibility_checks) {
+                           $bad_twin &&= $l>0 if $rule == $self;
 
                            if ($flags & Property::Flags::is_multiple_new &&
                                $l < $#$output &&
@@ -606,7 +615,7 @@ sub finalize {
                      }
                   }
                }
-               if ($Application::plausibility_checks && $bad_twin) {
+               if ($enable_plausibility_checks && $bad_twin) {
                   die "Twin property ", $ancestors[-1]->name, " occurs in all sources and in at least one target - rule should be flipped at ",
                       $self->source_location, "\n";
                }
@@ -619,7 +628,7 @@ sub finalize {
                splice @ancestors, 0, @ancestors-$pos;
             }
             if ($flags & Property::Flags::is_multiple) {
-               if ($Application::plausibility_checks) {
+               if ($enable_plausibility_checks) {
                   foreach ($common_prefix..($any_mult_depth || @$output)-2) {
                      if (($output->[$_]->flags & (Property::Flags::is_multiple | Property::Flags::is_multiple_new)) == Property::Flags::is_multiple) {
                         die "Multiple subobject ", $output->[$_]->name, " in path ", Property::print_path($output),
@@ -655,7 +664,7 @@ sub finalize {
       }
    } continue { ++$i }
 
-   if ($Application::plausibility_checks) {
+   if ($enable_plausibility_checks) {
       if (defined($perm_deputy) && !@{$perm_deputy->actions}) {
          die "None of the target properties is sensitive to the permutation ",
              Property::print_path($perm_deputy->perm_path), " which pretends to be triggered at ", $self->source_location, "\n";
@@ -679,16 +688,16 @@ sub finalize {
       $proto->add_rule_labels($perm_deputy, $self->labels) if defined $perm_deputy;
    }
 
-   if (defined $self->overridden_in) {
+   if (defined($self->overridden_in)) {
       $proto->override_rule(@$_) for @{$self->overridden_in};
       undef $self->overridden_in;
    }
 
    if (!$for_twin_rule && keys %twin_targets) {
       foreach my $twin_prop (keys %twin_targets) {
-         if (defined (my $flipped_rule=flip_rule_for_twin($self, $twin_prop))) {
+         if (defined(my $flipped_rule = flip_rule_for_twin($self, $twin_prop))) {
             push @{$proto->production_rules}, $flipped_rule;
-            finalize($flipped_rule, 1);
+            finalize($flipped_rule, true);
          }
       }
    }
@@ -708,43 +717,68 @@ sub prepare_header_search_pattern {
    $_[0] =~ s/([.|()])/\\$1/g;
 }
 
-# protected:
-# prepare a pattern for searching for rules by header
-sub header_search_pattern {
-   &prepare_header_search_pattern;
-   # expect optional labels omitted in the search pattern
-   $_[0] = qr/^(?: $hier_ids_re \s*:\s* )? (?-x:$_[0]) $/x;
+sub find_by_pattern {
+   my ($owner, $pattern, $filter) = @_;
+   if ($pattern =~ /^ (!)? ($hier_id_re) $/xo) {
+      # specified by label
+      my $label = $owner->find_rule_label($2)
+        or die "unknown label \"$pattern\"\n";
+      defined($filter)
+      ? grep { &$filter } $label->list_all_rules($1)
+      : $label->list_all_rules($1)
+
+   } elsif ($pattern =~ /:/) {
+      # specified by header
+      prepare_header_search_pattern($pattern);
+      # expect optional labels omitted in the search pattern
+      grep { $_->header =~ qr/^(?: $hier_ids_re \s*:\s* )? (?-x:$pattern) $/x } $owner->all_production_rules
+
+   } else {
+      croak( "invalid rule search pattern: expected a label or a complete header in the form \"OUTPUT : INPUT\"" );
+   }
 }
+
+sub all_to_disable {
+   my ($self) = @_;
+   if (defined(my $with_perm = $self->with_permutation)) {
+      ($self, $with_perm, @{$with_perm->actions})
+   } else {
+      ($self)
+   }
+}
+
 ####################################################################################
 # private:
-# create a closure working as a filter in Object::copy
+# create a closure working as a filter in BigObject::copy
 sub create_splitting_filter {
-   my ($rule, $permutation, $descend_path, $prop_hash, $sub_perm_hash, $depth)=@_;
+   my ($rule, $permutation, $descend_path, $prop_hash, $sub_perm_hash, $depth) = @_;
    sub {
-      my ($dst, $pv, $prop)=@_;
+      my ($dst, $pv, $prop) = @_;
       $prop //= $pv->property;
-      if (defined ($prop_hash) &&
-          defined (my $prod_list=$prop_hash->{$prop->key})) {
-         my $out=[ @$descend_path, $prop ];
+      my $recovery;
+      if (defined($prop_hash) &&
+          defined($recovery = $prop_hash->{$prop->key}) &&
+          $recovery != $Permutation::is_non_recoverable) {
+         my $out = [ @$descend_path, $prop ];
          push @{$rule->output}, $out;
-         my $action=new PermAction($rule, $#{$rule->output}, $prod_list, $depth, $depth && $permutation);
+         my $action = new PermAction($rule, $#{$rule->output}, $recovery, $depth, $depth && $permutation);
          push @{$rule->with_permutation->actions}, $action;
-         my $perm_dst=$dst->add_perm_in_parent($#$descend_path-$depth+1, $permutation, $rule);
+         my $perm_dst = $dst->add_perm_in_parent($#$descend_path-$depth+1, $permutation, $rule);
          local splice @$out, 0, $depth;
-         my ($perm_obj)=$perm_dst->descend_and_create($out);
+         my ($perm_obj) = $perm_dst->descend_and_create($out);
          return ($perm_obj, $pv->copy($perm_obj, $prop));
       }
 
-      if (defined ($sub_perm_hash) &&
-          defined (my $sub_permutation=$sub_perm_hash->{$prop->key})) {
+      if (defined($sub_perm_hash) &&
+          defined(my $sub_permutation = $sub_perm_hash->{$prop->key})) {
          return ($dst, $pv->copy($dst, $prop, create_splitting_filter($rule, $sub_permutation, [ @$descend_path, $prop ],
                                                                       $sub_permutation->sensitive_props, $sub_permutation->sub_permutations,
                                                                       $#$descend_path+2)));
       }
 
-      my $down_prop=$prop_hash->{$Permutation::sub_key};
+      my $down_prop = $prop_hash->{$Permutation::sub_key};
       $down_prop &&= $down_prop->{$prop->key};
-      my $down_sub=$sub_perm_hash->{$Permutation::sub_key};
+      my $down_sub = $sub_perm_hash->{$Permutation::sub_key};
       $down_sub &&= $down_sub->{$prop->key};
       if (defined($down_prop) || defined($down_sub)) {
          return ($dst, $pv->copy($dst, $prop, create_splitting_filter($rule, $permutation, [ @$descend_path, $prop ],
@@ -757,7 +791,7 @@ sub create_splitting_filter {
 
 sub permuting {
    my ($pkg, $proto, $perm_path) = @_;
-   my $self=_new($pkg, 'copying', undef, $proto);
+   my $self = _new($pkg, 'copying', undef, $proto);
    $self->weight = $zero_weight;
    $self->with_permutation = new FakeCreatingPermutation($self, $perm_path);
    my $permutation = local pop @$perm_path;
@@ -787,59 +821,42 @@ sub create {
    $self
 }
 ####################################################################################
-my $break_reason;
-
-sub break_rule {
-   $break_reason = shift;
-   if ($break_reason eq 'ALRM') {
-      $SIG{INT} = 'IGNORE';
-      $SIG{ALRM} = 'IGNORE';
-      kill -(SIGINT), $$;       # kill the subprocesses
-   }
-   die "\n" if waitpid(-$$,WNOHANG)==-1;        # there were no subprocesses - leave eval{} in execute().
-}
-
-my $breaksignals=new POSIX::SigSet(SIGINT, SIGALRM);
-my $sa_break=new POSIX::SigAction(\&break_rule, $breaksignals, 0);
-my $sa_INT_save=new POSIX::SigAction('IGNORE');
 
 # Execute the rule on a separate transaction level
 sub execute {
+   local interrupts(block);
    my ($self, $object, $force) = @_;
    unless ($self->flags & Flags::is_precondition) {
-      my $trans = new RuleTransaction($object, $self);
+      my $trans = new RuleTransaction($object, $self, $force==2);
       unless ($force || defined($trans->changed)) {
          $trans->rollback($object);
          return Exec::OK;
       }
    }
    dbg_print("applying rule ", $self->header) if $Verbose::rules>2;
-   &_execute;
+   &execute_me;
 }
 
 # private:
-sub _execute {
+sub execute_me {
    my ($self, $object) = @_;
-   my $died = 1;
    my ($rc, $retval) = (Exec::failed);
-   undef $break_reason;
    eval {
-      sigaction SIGINT, $sa_break, $sa_INT_save;
-      sigaction SIGALRM, $sa_break;
-      my $alarm_time = $timeout && !($self->flags & Flags::is_precondition);
-      alarm $alarm_time if $alarm_time;
+      ## my $alarm_time = $timeout && !($self->flags & Flags::is_precondition);
+      ## alarm $alarm_time if $alarm_time;
+      local interrupts(enable);
       if (wantarray || $self->flags & Flags::is_precondition) {
          $retval=$self->code->($object);
       } else {
          # call production rules in void context
          $self->code->($object);
       }
-      alarm 0 if $alarm_time;
+      ## alarm 0 if $alarm_time;
       if ($self->flags & Flags::is_precondition) {
          $rc= $retval ? Exec::OK : Exec::failed;
-         if ($Verbose::rules>1) {
+         if ($Verbose::rules > 1) {
             if ($retval) {
-               dbg_print( $self->header, " satisfied" ) if $self->flags & Flags::is_precondition and $Verbose::rules>2;
+               dbg_print( $self->header, " satisfied" ) if $self->flags & Flags::is_precondition and $Verbose::rules > 2;
             } else {
                warn_print( $self->header, $self->flags & Flags::is_precondition ? " not satisfied" : " failed" );
             }
@@ -850,22 +867,21 @@ sub _execute {
          $rc = Exec::OK;
       }
       $object->commit unless ($self->flags & Flags::is_precondition);
-      $died=0;
    };
-   sigaction SIGINT, $sa_INT_save;
-   $SIG{ALRM}='IGNORE';
-
-   if ($died) {
-      $rc = Exec::failed;
-      if ($break_reason eq "ALRM") {
-         $@ = "timeout elapsed\n";
-      } elsif ($break_reason eq "INT") {
-         $@ = "killed by signal\n";
-         $rc = Exec::retry;
-      }
-      $object->failed_rules->{$self}=1;
-      $object->failed_rules->{$self->with_permutation}=1 if defined $self->with_permutation;
+   if ($@) {
       $object->rollback unless ($self->flags & Flags::is_precondition);
+      if ($@ eq "Interrupted\n") {
+         die $@;
+      }
+      if ($@ eq "Infeasible\n") {
+         $rc = Exec::infeasible;
+      } else {
+         $rc = Exec::failed;
+         $object->failed_rules->{$self} = 1;
+         if (defined(my $with_perm = $self->with_permutation)) {
+            $object->failed_rules->{$_} = 1 for $with_perm, @{$with_perm->actions};
+         }
+      }
    }
    wantarray ? ($rc, $retval) : $rc;
 }
@@ -928,15 +944,17 @@ sub descend_to_source {
 }
 
 sub execute {
-   my ($self, $object)=@_;
-   dbg_print("applying rule ", $self->header) if $Verbose::rules>2;
-   if (defined (my $pv=&descend_to_source)) {
-      my $trans=$object->begin_transaction;
-      my ($subobj, $prop)=$object->descend_and_create($self->output->[0]);
-      my $ref_pv=new PropertyValue($prop, ($pv!=$object) ? ($pv->value, defined($pv->value) && PropertyValue::Flags::is_strong_ref)
-                                                         : ($object, PropertyValue::Flags::is_weak_ref));
+   local interrupts(block);
+   my ($self, $object) = @_;
+   dbg_print("applying rule ", $self->header) if $Verbose::rules > 2;
+   if (defined(my $pv = &descend_to_source)) {
+      my $trans = $object->begin_transaction;
+      my ($subobj, $prop) = $object->descend_and_create($self->output->[0]);
+      my $ref_pv = new PropertyValue($prop,
+         $pv != $object ? ($pv->value, defined($pv->value) && PropertyValue::Flags::is_shortcut)
+                        : ($object, PropertyValue::Flags::is_weak_ref));
       push @{$subobj->contents}, $ref_pv;
-      $subobj->dictionary->{$prop->key}=$#{$subobj->contents};
+      $subobj->dictionary->{$prop->key} = $#{$subobj->contents};
       $trans->commit($object);
       wantarray ? (Exec::OK, $ref_pv) : Exec::OK;
    } else {
@@ -1006,10 +1024,11 @@ sub with_permutation { undef }
 sub without_permutation { $_[0]->rule }
 
 sub execute {
-   my ($self, $object)=@_;
-   new RuleTransactionWithPerm($object, $self);
+   local interrupts(block);
+   my ($self, $object, $force) = @_;
+   new RuleTransactionWithPerm($object, $self, $force==2);
    dbg_print("applying rule ", $self->header) if $Verbose::rules>2;
-   _execute($self->rule, $object);
+   execute_me($self->rule, $object);
 }
 
 sub record_sensitivity {
@@ -1019,7 +1038,7 @@ sub record_sensitivity {
    foreach my $action (@{$self->actions}) {
       if ($action->depth) {
          my $out=$self->rule->output->[$action->output];
-         local splice @$out, $action->depth + 1;
+         local splice @$out, $action->depth+1;
          my ($sub_obj)=$object->descend($out);
          $sub_obj->sensitive_to->{$action->sub_permutation->key}=1;
       }
@@ -1031,7 +1050,7 @@ package Polymake::Core::Rule::FakeCreatingPermutation;
 use Polymake::Struct [ '@ISA' => 'CreatingPermutation' ];
 
 sub execute {
-   my $object=$_[1];
+   my $object = $_[1];
    $object->commit;
    &record_sensitivity;
    Exec::OK
@@ -1041,15 +1060,15 @@ package Polymake::Core::Rule::PermAction;
 
 use Polymake::Struct (
    [ new => '$$$$;$' ],
-   [ '$perm_trigger' => 'weak(#1)' ], # production rule triggering the permutation
+   [ '$perm_trigger' => 'weak(#1)' ],   # production rule triggering the permutation
    [ '$output' => '#2' ],               # index into its output where the sensitive property lies
-   [ '$producers' => '#3' ],            # ( Rule ) moving the output property back to the basis
+   [ '$recovery' => '#3' ],             # Permutation::Recovery to obtain Rules moving the output property back to the basis
    [ '$depth' => '#4' ],                # how deep to descend in the base object
    [ '$sub_permutation' => '#5' ],      # Permutation for the subobject
 );
 
 sub header {
-   my $self=shift;
+   my ($self) = @_;
    Property::print_path($self->perm_trigger->output->[$self->output])
    . " extracted from " . Property::print_path($self->perm_path) . " after " . $self->perm_trigger->header;
 }
@@ -1057,7 +1076,7 @@ sub list_results { () }
 sub perm_path { $_[0]->perm_trigger->with_permutation->perm_path }
 sub flags { Flags::is_perm_action }
 sub weight { $zero_weight }
-sub enabled { @{$_[0]->producers}!=0 }
+sub enabled { $_[0]->recovery != $Permutation::is_non_recoverable }
 
 sub preconditions { [ ] }
 sub with_permutation { undef }
@@ -1082,8 +1101,8 @@ sub execute {
 }
 
 sub find_sensitive_sub_property {
-   my ($self, @path)=@_;
-   if (Property::equal_path_prefixes($self, \@path)==$#$self) {
+   my ($self, @path) = @_;
+   if (Property::equal_path_prefixes($self, \@path) == $#$self) {
       $self->[-1]->find_sensitive_sub_property(splice @path, $#$self)
    } else {
       undef
